"""
Base class and utilities for TGN Python Tcl wrapper.

@author: yoram.shamir
"""

import sys
from os import path
import logging
import time
import re
from threading import Thread
from queue import Queue

from trafficgenerator.tgn_utils import new_log_file

# IxExplorer only uses Tcl utilities (over socket) without Tcl interpreter so it's OK if Tcl is not installed (e.g for
# some Linux installations). If Tcl interpreter is required and not installed it will fail anyway...
try:
    if sys.version_info[0] < 3:
        from Tkinter import Tcl
    else:
        from tkinter import Tcl
except Exception as _:
    pass


def tcl_str(string=''):
    """
    :param string: Python string.
    :returns: Tcl string surrounded by {}.
    """

    return ' {' + string + '} '


def tcl_file_name(name):
    """
    :param names: file name.
    :returns: normalized file name with forward slashes.
    """

    return tcl_str(path.normpath(name).replace('\\', '/'))


def get_args_pairs(arguments):
    """
    :param arguments: Python dictionary of TGN API command arguments <key, value>.
    :returns: Tcl list of argument pairs <-key, value> to be used in TGN API commands.
    """

    return ' '.join(' '.join(['-' + k, tcl_str(str(v))]) for k, v in arguments.items())


def build_obj_ref_list(objects):
    """
    :param objects: Python list of requested objects.
    :returns: Tcl list of all requested objects references.
    """

    return ' '.join([o.obj_ref() for o in objects])


tcl_interp_g = None
""" Global Tcl interpreter for Tcl based utilities. Does not log its operations. """


def tcl_list_2_py_list(tcl_list, within_tcl_str=False):
    """ Convert Tcl list to Python list using Tcl interpreter.

    :param tcl_list: string representing the Tcl string.
    :param within_tcl_str: True - Tcl list is embedded within Tcl str. False - native Tcl string.
    :return: Python list equivalent to the Tcl ist.
    :rtye: list
    """

    if not within_tcl_str:
        tcl_list = tcl_str(tcl_list)
    return tcl_interp_g.eval('join ' + tcl_list + ' LiStSeP').split('LiStSeP') if tcl_list else []


def py_list_to_tcl_list(py_list):
    """ Convert Python list to Tcl list using Tcl interpreter.

    :param py_list: Python list.
    :type py_list: list
    :return: string representing the Tcl string equivalent to the Python list.
    """

    py_list_str = [str(s) for s in py_list]
    return tcl_str(tcl_interp_g.eval('split' + tcl_str('\t'.join(py_list_str)) + '\\t'))


class TgnTk(object):
    """ Native Python Tk interpreter. """

    def __init__(self):
        self.tcl = Tcl()

    def eval(self, command):
        return self.tcl.eval(command)


class TgnTkMultithread(Thread):
    """ Native Python Tk interpreter with multithreading. """

    _is_running = True

    def __init__(self):
        super(self.__class__, self).__init__()
        self.in_q = Queue()
        self.out_q = Queue()
        self.tcl = None

    def run(self):
        if not self.tcl:
            self.tcl = Tcl()
        while self._is_running:
            if not self.in_q.empty():
                command = self.in_q.get()
                try:
                    rc = self.tcl.eval(command)
                    self.out_q.put(rc)
                except Exception as e:
                    self.out_q.put(e)
            time.sleep(1)

    def stop(self):
        self._is_running = False

    def eval(self, command):
        self.in_q.put(command)
        while self.out_q.empty():
            time.sleep(1)
        rc = self.out_q.get()
        if isinstance(rc, Exception):
            raise rc
        return rc


class TgnTclConsole(object):
    """ Tcl interpreter over console.

    Current implementation is a sample extracted from actual project where the console is telnet to Windows machine.
    """

    def __init__(self, con, tcl_exe):
        """ Start Tcl interpreter on console.

        :param con: console.
        :param tcl_exe: full path to Tcl exe.
        """
        super(TgnTclConsole, self).__init__()
        self._con = con
        self._con.set_prompt_match_expression('% ')
        self._con.send_cmd(tcl_exe)

    def eval(self, command):
        """
        @summary: Evaluate Tcl command.

        @param command: command to evaluate.
        @return: command output.
        """
        # Some operations (like take ownership) may take long time.
        con_command_out = self._con.send_cmd(command, timeout=256)
        if 'ERROR_SEND_CMD_EXIT_DUE_TO_TIMEOUT' in con_command_out:
            raise Exception('{} - command timeout'.format(command))
        command = command.replace('\\', '/')
        con_command_out = con_command_out.replace('\\', '/')
        command = command.replace('(', '\(').replace(')', '\)')
        command = command.replace('{', '\{').replace('}', '\}')
        m = re.search(command + '(.*)' + '%', con_command_out, re.DOTALL)
        command_out = m.group(1).strip()
        if 'couldn\'t read file' in command_out or 'RuntimeError' in command_out:
            raise Exception(command_out)
        return command_out

    def disconnect(self):
        self._con.set_prompt_match_expression('C:.*>')
        self._con.send_cmd('exit')


class TgnTclWrapper(object):
    """ Tcl connectivity for TGN projects. """

    def __init__(self, logger, tcl_interp=None):
        """ Init Python Tk package.

        Add logger to log Tcl commands only.
        This creates a clean Tcl script that can be used later for debug.
        We assume that there might have both multiple Tcl sessions simultaneously so we add suffix to create
        multiple distinguished Tcl scripts.
        """

        if not logger:
            logger = logging.getLogger('dummy')
        self.logger = logger
        self.tcl_script = new_log_file(self.logger, self.__class__.__name__)

        if not tcl_interp:
            self.tcl_interp = TgnTk()
        else:
            self.tcl_interp = tcl_interp
        global tcl_interp_g
        tcl_interp_g = self.tcl_interp

    def eval(self, command):
        """ Execute Tcl command.

        Write the command to tcl script (.tcl) log file.
        Execute the command.
        Write the command and the output to general (.txt) log file.

        :param command: Command to execute.
        :returns: command raw output.
        """

        if self.logger.handlers:
            self.logger.debug(command)
        if self.tcl_script:
            self.tcl_script.info(command)
        self.rc = self.tcl_interp.eval(command)
        if self.logger.handlers:
            self.logger.debug('\t' + self.rc)
        return self.rc

    def source(self, script_file):
        self.eval('source ' + tcl_file_name(script_file))



"""
Tests for basic TGN object operations.

@author yoram@ignissoft.com
"""

import unittest
from mock import MagicMock

from trafficgenerator.tgn_utils import is_false, is_true, is_local_host, is_ip, flatten, TgnError
from trafficgenerator.tgn_object import TgnObject, TgnObjectsDict


class TgnObjectTest(unittest.TestCase):

    def setUp(self):
        self.root = TgnObject(objRef='root1', objType='root', parent=None)
        self.root.api = None
        self.root.logger = None
        self.leaf1 = TgnObject(objRef='leaf1', objType='leaf', parent=self.root)
        self.node1 = TgnObject(objRef='node1', objType='node', parent=self.root, name='name1')
        self.node2 = TgnObject(objRef='node2', objType='node', parent=self.root, name='name2')
        self.node11 = TgnObject(objRef='node11', objType='node', parent=self.node1, name='name11')
        self.node12 = TgnObject(objRef='node12', objType='node', parent=self.node1, name='name12')
        self.leaf11 = TgnObject(objRef='leaf11', objType='leaf', parent=self.node1)
        for o in self.__dict__.values():
            if type(o) == TgnObject:
                self._mock_object(o)

    def tearDown(self):
        pass

    def testHelloWorld(self):
        pass

    def testObjectsTree(self):
        """ Test object search operations. """

        assert(self.root.obj_ref() == 'root1')
        assert(self.root.obj_type() == 'root')
        assert(self.root.obj_name() == 'root1')
        assert(self.node1.obj_ref() == 'node1')
        assert(self.node1.obj_type() == 'node')
        assert(self.node1.obj_name() == 'name1')
        assert(self.node1.obj_parent() == self.root)

        assert(self.root.get_object_by_name('name2') == self.node2)
        assert(len(self.root.get_objects_by_type('node')) == 2)
        assert(len(self.root.get_objects_by_type('no_such_object')) == 0)
        assert(self.root.get_object_by_ref('leaf1') == self.leaf1)

        assert(len(self.root.get_objects_by_type_in_subtree('node')) == 4)
        assert(len(self.root.get_objects_by_type_in_subtree('leaf')) == 2)
        assert(len(self.node11.get_objects_by_type_in_subtree('node')) == 0)

        assert(str(self.root) == self.root.obj_name())

        assert(len(self.root.get_objects_with_attribute('node', 'attr_name', 'node1')) == 1)

    def _mock_object(self, o):
        o.get_attribute = MagicMock(name='get_attribute')
        o.get_attribute('attr_name')
        o.get_attribute.return_value = o.obj_ref()

    def test_objects_dict(self):
        objects_dict = TgnObjectsDict()
        objects_dict[self.node1] = TgnObjectsDict()
        objects_dict[self.node1][self.node11] = 'node 11 entry'
        objects_dict[self.node1][self.node12] = 'node 12 entry'
        objects_dict[self.node2] = 'node 2 entry'
        self.assertRaises(TgnError, objects_dict.__setitem__, 'invalid key', '')
        assert(objects_dict[self.node2] == 'node 2 entry')
        assert(objects_dict[self.node2.name] == 'node 2 entry')
        assert(objects_dict[self.node2.ref] == 'node 2 entry')
        print(objects_dict.dumps())


class TgnUtilsTest(unittest.TestCase):

    def testTrueFalse(self):
        """ Test TGN true and false values. """

        for false_stc in ('False', 'false', '0', 'null', 'NONE', 'none', '::ixnet::obj-null'):
            assert(is_false(false_stc))
            assert(not is_true(false_stc))

        for true_str in ('True', 'TRUE', '1'):
            assert(is_true(true_str))
            assert(not is_false(true_str))

    def testLocalhost(self):
        """ Test TGN localhost values. """

        for location in ('127.0.0.1', 'localhost', 'Localhost/1/1', '//(Offline)/1/1', 'null'):
            assert(is_local_host(location))

        for location in ('1.2.3.4', 'hostname', '192.168.1.1/1/2'):
            assert(not is_local_host(location))

    def testIps(self):
        """ Test TGN IP values. """

        for ip in ('IPV4', 'ipv6', 'ipv4if', 'IPV6IF'):
            assert(is_ip(ip))

        for ip in ('mac', 'bla'):
            assert(not is_ip(ip))

    def testFlatten(self):
        nl = [1, [11, [111]], 2, [22]]
        assert(len(nl) == 4)
        assert(type(nl[1]) == list)
        assert(type(nl[2]) == int)
        assert(len(flatten(nl)) == 5)
        assert(type(flatten(nl)[1]) == int)
        assert(type(flatten(nl)[2]) == int)



"""
Tests for TGN Tcl multithreading interpreter.

@author yoram@ignissoft.com
"""

import sys
from os import path
import unittest
import logging
from configparser import SafeConfigParser

from trafficgenerator.tgn_tcl import TgnTkMultithread, TgnTclWrapper

config_file = path.join(path.dirname(__file__), 'TrafficGenerator.ini')


class TclTest(unittest.TestCase):

    def setUp(self):
        global tcl

        config = SafeConfigParser(allow_no_value=True)
        config.read(config_file)

        logger = logging.getLogger('log')
        logger.setLevel(config.get('Logging', 'level'))
        logger.addHandler(logging.FileHandler(config.get('Logging', 'file_name')))
        logger.addHandler(logging.StreamHandler(sys.stdout))

        self.tcl_interp = TgnTkMultithread()
        self.tcl_interp.start()
        self.tcl = TgnTclWrapper(logger, self.tcl_interp)

    def tearDown(self):
        self.tcl_interp.stop()

    def testPuts(self):
        print(self.tcl.eval('set dummy "hello world"'))



"""
TGN projects utilities and errors.

@author: yoram.shamir
"""

import logging
from os import path
from enum import Enum
import collections


class TgnType(Enum):
    ixexplorer = 1
    ixnetwork = 2
    testcenter = 3


class ApiType(Enum):
    tcl = 1
    python = 2
    rest = 3
    socket = 4


def flatten(x):
    if isinstance(x, collections.Iterable):
        return [a for i in x for a in flatten(i)]
    else:
        return [x]


def is_true(str_value):
    """
    :param str_value: String to evaluate.
    :returns: True if string represents True TGN attribute value else return False.
    """
    return str_value.lower() in ('true', '1', '::ixnet::ok')


def is_false(str_value):
    """
    :param str_value: String to evaluate.
    :returns: True if string represents TGN attribute False value else return True.
    """
    return str_value.lower() in ('false', '0', 'null', 'none', '::ixnet::obj-null')


def is_local_host(location):
    """
    :param location: Location string in the format ip[/slot[/port]].
    :returns: True if ip represents localhost or offilne else return False.
    """
    return any(x in location.lower() for x in ('localhost', '127.0.0.1', 'offline', 'null'))


def is_ipv4(str_value):
    """
    :param str_value: String to evaluate.
    :returns: True if string represents IPv4 else return False.
    """
    return str_value.lower() in ('ipv4', 'ipv4if')


def is_ipv6(str_value):
    """
    :param str_value: String to evaluate.
    :returns: True if string represents IPv6 else return False.
    """
    return str_value.lower() in ('ipv6', 'ipv6if')


def is_ip(str_value):
    """
    :param str_value: String to evaluate.
    :returns: True if string is IPv4 or IPv6 else return False.
    """
    return is_ipv4(str_value) or is_ipv6(str_value)


def new_log_file(logger, suffix, file_type='tcl'):
    """ Create new logger and log file from existing logger.

    The new logger will be create in the same directory as the existing logger file and will be named
    as the existing log file with the requested suffix.

    :param logger: existing logger
    :param suffix: string to add to the existing log file name to create the new log file name.
    :param file_type: logger file type (tcl. txt. etc.)
    :return: the newly created logger
    """

    file_handler = None
    for handler in logger.handlers:
        if isinstance(handler, logging.FileHandler):
            file_handler = handler
    new_logger = logging.getLogger(file_type + suffix)
    if file_handler:
        logger_file_name = path.splitext(file_handler.baseFilename)[0]
        tcl_logger_file_name = logger_file_name + '-' + suffix + '.' + file_type
        new_logger.addHandler(logging.FileHandler(tcl_logger_file_name, 'w'))
        new_logger.setLevel(logger.getEffectiveLevel())

    return new_logger


class TgnError(Exception):
    """ Base exception for traffic generator exceptions. """
    pass



"""
Base classes and utilities for TGN applications classes.

@author yoram@ignissoft.com
"""


class TgnApp(object):
    """ Base class for all TGN applications classes. """

    def __init__(self, logger, api_wrapper):
        self.logger = logger
        self.api = api_wrapper



"""
Base class and utilities for all TGN objects.

@author yoram@ignissoft.com
"""

from collections import OrderedDict
import gc
from abc import ABCMeta, abstractmethod
import json

from trafficgenerator.tgn_utils import TgnError


# Workaround IXN object reference bugs.
# Object reference with float sequential number instead of integer.
#    For example, endpointset->sources attribute might return:
#    vport:1/protocols/bgp/neighborRange:1.0/routeRange:1.
# Object reference with neighborPairs (plural) instead of neighborPair (single).
def _WA_norm_obj_ref(obj_ref):
    return obj_ref.replace('.0', '').replace('neighborPairs:', 'neighborPair:')


class TgnObjectsDict(OrderedDict):
    """ Dictionary to map from TgnObjects to whatever data.

    Dictionary keys must be TgnObject but then it can be accessed by the object itself, the object reference or the
    object name.
    """

    def __setitem__(self, key, value):
        if not isinstance(key, TgnObject):
            raise TgnError('tgn_object_dict keys must be TgnObject, not {}'.format(type(key)))
        return OrderedDict.__setitem__(self, key, value)

    def __getitem__(self, key):
        if key in self.keys():
            return OrderedDict.__getitem__(self, key)
        else:
            for obj in self:
                if obj.name == key or obj.ref == key:
                    return OrderedDict.__getitem__(self, obj)

    def dumps(self, indent=1):
        """ Returns nested string representation of the dictionary (like json.dumps).

        :param indent: indentation level.
        """
        str_keys_dict = OrderedDict({str(k): v for k, v in self.items()})
        for k, v in str_keys_dict.items():
            if isinstance(v, dict):
                str_keys_dict[k] = OrderedDict({str(k1): v1 for k1, v1 in v.items()})
                for k1, v1 in str_keys_dict[k].items():
                    if isinstance(v1, dict):
                        str_keys_dict[k][k1] = OrderedDict({str(k2): v2 for k2, v2 in v1.items()})
        return json.dumps(str_keys_dict, indent=indent)


class TgnObject(object):
    """ Base class for all TGN classes. """

    objects = {}
    """ Dictionary of child objects <object reference: object name>. """

    def __init__(self, **data):
        """ Create new TGN object in the API.

        If object does not exist on the chassis, create it on the chassis as well.

        :param parent: object parent. If == None the api and logger attributes must be set explicitly by the caller.
        """

        super(TgnObject, self).__init__()
        self._data = {}
        self.objects = OrderedDict()
        self._set_data(**data)
        if self._data['parent']:
            self.api = self.obj_parent().api
            self.logger = self.obj_parent().logger
        if 'objRef' not in self._data:
            self._data['objRef'] = self._create()
        if 'name' not in self._data:
            self._data['name'] = self.obj_ref()
        if self._data.get('parent', None):
            # todo: make sure each object has parent and test only for None parents (STC project and IXN root)..
            self._data['parent'].objects[self.obj_ref()] = self

    def __str__(self):
        return self.name

    def get_subtree(self, types=[], level=1):
        """ Read all direct children of the requested types and all their descendants down to the requested level.

        :param types: list of requested types.
        :param level: how many levels to go down the subtree.
        """

        if level == 0:
            return
        for child in self.get_children(*types):
            child.get_subtree(level=level - 1)

    def get_child(self, *types):
        """
        :param types: list of requested types.
        :return: the first (and in most useful cases only) child of specific type(s).
        """
        children = self.get_children(*types)
        return children[0] if any(children) else None

    def get_object_by_ref(self, obj_ref):
        """
        :param obj_ref: requested object reference.
        :return: the first object with the requested object reference in the object branch.
        """
        return self._get_object_by_key('objRef', _WA_norm_obj_ref(obj_ref))

    def get_object_by_name(self, obj_name):
        """
        :param obj_name: requested object name.
        :return: the first object with the requested object name in the object branch.
        """
        return self._get_object_by_key('name', obj_name)

    def _get_object_by_key(self, key, value, *types):
        if self._data[key] == value and (types and self.obj_type() in types or not types):
            return self
        else:
            if not types:
                children = self.objects.values()
            else:
                children = self.get_objects_by_type(*types)
            for child in children:
                obj = child._get_object_by_key(key, value, *types)
                if obj is not None:
                    return obj

    def get_objects_by_type(self, *types):
        """ Returned objects stored in memory (without re-reading them from the TGN).

        Use this method for fast access to objects in case of static configurations.

        :param types: requested object types.
        :return: all children of the specified types.
        """

        if not types:
            return self.objects.values()
        types_l = [o.lower() for o in types]
        return [o for o in self.objects.values() if o.obj_type().lower() in types_l]

    def get_object_by_type(self, *types):

        """
        :param types: requested object types.
        :return: the child of the specified types.
        """
        children = self.get_objects_by_type(*types)
        return children[0] if any(children) else None

    def get_objects_by_type_in_subtree(self, *types):
        """
        :param types: requested object types.
        :return: all children of the specified types.
        """

        typed_objects = self.get_objects_by_type(*types)
        for child in self.objects.values():
            typed_objects += child.get_objects_by_type_in_subtree(*types)
        return typed_objects

    def get_objects_or_children_by_type(self, *types):
        """ Get objects if children already been read or get children.

        Use this method for fast access to objects in case of static configurations.

        :param types: requested object types.
        :return: all children of the specified types.
        """

        objects = self.get_objects_by_type(*types)
        return objects if objects else self.get_children(*types)

    def get_objects_with_object(self, obj_type, *child_types):
        """
        :param obj_type: requested object type.
        :param child_type: requested child types.
        :return: all children of the requested type that have the requested child types.
        """

        return [o for o in self.get_objects_by_type(obj_type) if
                o.get_objects_by_type(*child_types)]

    def get_objects_without_object(self, obj_type, *child_types):
        """
        :param obj_type: requested object type.
        :param child_type: unrequested child types.
        :return: all children of the requested type that do not have the unrequested child types.
        """
        return [o for o in self.get_objects_by_type(obj_type) if
                not o.get_objects_by_type(*child_types)]

    def get_objects_with_attribute(self, obj_type, attribute, value):
        """
        :param obj_type: requested object type.
        :param attribute: requested attribute.
        :param value: requested attribute value.
        :return: all children of the requested type that have the requested attribute == requested value.
        """
        return [o for o in self.get_objects_by_type(obj_type) if o.get_attribute(attribute) == value]

    def get_ancestor_object_by_type(self, obj_type):
        """
        :param obj_type: requested ancestor type.
        :return: the ancestor of the object who's type is obj_type if exists else None.
        """

        if self.type.lower() == obj_type.lower():
            return self
        else:
            if not self.parent:
                return None
            return self.parent.get_ancestor_object_by_type(obj_type)

    def get_object_from_attribute(self, attribute):
        return self.get_objects_from_attribute(attribute)[0] if self.get_objects_from_attribute(attribute) else None

    @abstractmethod
    def get_objects_from_attribute(self, attribute):
        pass

    def del_object_from_parent(self):
        """ Delete object from parent object. """
        if self.parent:
            self.parent.objects.pop(self.ref)

    def del_objects_by_type(self, type_):
        """ Delete all children objects.

        :param type_: type of objects to delete.
        """
        [o.del_object_from_parent() for o in self.get_objects_by_type(type_)]

    @classmethod
    def get_objects_of_class(cls):
        """
        :return: all instances of the requested class.
        """
        return list(o for o in gc.get_objects() if isinstance(o, cls))

    #
    # Simple utilities to return object _data. Maybe it's not Pythonic (more like Java) but after
    # changing the key name couple of times I decided to go for it.
    #

    def obj_name(self):
        """
        :return: object name.
        """
        return self._data['name']
    name = property(obj_name)

    def obj_ref(self):
        """
        :return: object reference.
        """
        return str(self._data['objRef'])
    ref = property(obj_ref)

    def obj_type(self):
        """
        :return: object type.
        """
        return self._data['objType']
    type = property(obj_type)

    def obj_parent(self):
        """
        :return: object parent.
        """
        return self._data['parent']
    parent = property(obj_parent)

    #
    # Private methods.
    #

    def _set_data(self, **data):
        self._data.update(data)

    def _build_children_objs(self, child_type, children):
        children_objs = OrderedDict()
        child_obj_type = self.get_obj_class(child_type)
        for child in (c for c in children if c is not ''):
            child_object = child_obj_type(objRef=child, objType=child_type, parent=self)
            child_object._set_data(name=child_object.get_name())
            children_objs[child_object.obj_ref()] = child_object
        self.objects.update(children_objs)
        return children_objs

    #
    # Abstract API methods.
    #

    @abstractmethod
    def get_attribute(self, attribute):
        """ Get single attribute value.

        :param attribute: attribute name.
        :return: attribute value.
        """
        pass

    @abstractmethod
    def get_children(self, *types):
        """ Get all children of the requested types.

        :param attribute: requested children types.
        :return: list of all children of the requested types.
        """
        pass


class TgnL3(object):
    """ ABC for all L3 objects. """

    __metaclass__ = ABCMeta

    @abstractmethod
    def ip(self):
        """
        :return: IP address.
        """
        pass

    @abstractmethod
    def num_ips(self):
        """
        :return: number of IP addresses.
        """
        pass



#!/usr/bin/env python
# encoding: utf-8

from __future__ import print_function
from setuptools import setup
import io

import trafficgenerator


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


long_description = read('README.txt')

with open('requirements.txt') as f:
    required = f.read().splitlines()
install_requires = [r for r in required if r and r[0] != '#' and not r.startswith('git')]

setup(
    name='pytrafficgen',
    version=trafficgenerator.__version__,
    url='https://github.com/shmir/PyTrafficGenerator/',
    license='Apache Software License',
    author='Yoram Shamir',
    install_requires=install_requires,
    author_email='yoram@ignissoft.com',
    description='Base Python OO API package to automate traffic generators (Spirent TestCenter, Ixia IxNetwork etc.)',
    long_description=long_description,
    packages=['trafficgenerator', 'trafficgenerator.test'],
    include_package_data=True,
    platforms='any',
    tests_require=['pytest'],
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Testing :: Traffic Generation'],
)



"""
Base class for all traffic generators tests.

@author yoram@ignissoft.com
"""

from os import path
import sys
import unittest
import logging
from configparser import SafeConfigParser


class TgnTest(unittest.TestCase):
    """ Base class for all TGN tests - read ini file and create logger. """

    config_file = path.join(path.dirname(__file__), 'TrafficGenerator.ini')

    config = None
    logger = logging.getLogger('log')

    @classmethod
    def setUpClass(cls):
        TgnTest.config = SafeConfigParser(allow_no_value=True)
        TgnTest.config.read(TgnTest.config_file)

        TgnTest.logger.setLevel(TgnTest.config.get('Logging', 'level'))
        if TgnTest.config.get('Logging', 'file_name'):
            TgnTest.logger.addHandler(logging.FileHandler(TgnTest.config.get('Logging', 'file_name')))
        TgnTest.logger.addHandler(logging.StreamHandler(sys.stdout))

    @classmethod
    def tearDownClass(cls):
        pass

    def testHelloWorld(self):
        print(sys.version)



"""
Tests for TGN Tcl wrapper - the default wrapper.

@author yoram@ignissoft.com
"""

import sys
from os import path
import unittest
import logging
from configparser import SafeConfigParser

from trafficgenerator.tgn_tcl import TgnTclWrapper, tcl_list_2_py_list, py_list_to_tcl_list, tcl_file_name

config_file = path.join(path.dirname(__file__), 'TrafficGenerator.ini')


class TclTest(unittest.TestCase):

    def setUp(self):
        config = SafeConfigParser(allow_no_value=True)
        config.read(config_file)

        logger = logging.getLogger('log')
        logger.setLevel(config.get('Logging', 'level'))
        logger.addHandler(logging.FileHandler(config.get('Logging', 'file_name')))
        logger.addHandler(logging.StreamHandler(sys.stdout))

        self.tcl = TgnTclWrapper(logger)

    def tearDown(self):
        pass

    def testList(self):
        """ Test Python->Tcl and Tcl->Python list conversion. """

        py_list = ['a', 'b b']
        tcl_list_length = self.tcl.eval('llength ' + py_list_to_tcl_list(py_list))
        assert(int(tcl_list_length) == 2)

        tcl_list = '{a} {b b}'
        py_list_length = len(tcl_list_2_py_list(tcl_list))
        assert(py_list_length == 2)

    def testFilename(self):
        """ Test Tcl file names normalization. """

        assert(tcl_file_name('a\\b/c').strip() == '{a/b/c}')
